/*
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2003 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Cactus" and "Apache Software
 *    Foundation" must not be used to endorse or promote products
 *    derived from this software without prior written permission. For
 *    written permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */
package org.apache.cactus.integration.ant;

import org.apache.tools.ant.BuildException;

/**
 * Unit tests for {@link RunServerTestsTask}.
 * 
 * @author <a href="mailto:cmlenz@apache.org">Christopher Lenz</a>
 *
 * @version $Id: TestRunServerTestsTask.java,v 1.4 2003/06/09 12:35:57 cmlenz Exp $
 */
public final class TestRunServerTestsTask extends AntTestCase
{

    // Constructors ------------------------------------------------------------

    /**
     * @see AntTestCase#AntTestCase
     */
    public TestRunServerTestsTask()
    {
        super("org/apache/cactus/integration/ant/test-runservertests.xml");
    }

    // TestCase Implementation -------------------------------------------------

    /**
     * @see junit.framework.TestCase#setUp()
     */
    protected void setUp() throws Exception
    {
        super.setUp();

        getProject().addTaskDefinition("runservertests",
            RunServerTestsTask.class);
    }

    // Test Methods ------------------------------------------------------------

    /**
     * Verifies that a build exception is thrown if neither the
     * <code>starttarget</code> nor a nested <code>start</code> element has been
     * specified.
     * 
     * @throws Exception If an unexpected error occurs
     */
    public void testStartNotSet() throws Exception
    {
        try
        {
            executeTestTarget();
            fail("Expected BuildException");
        }
        catch (BuildException expected)
        {
            assertEquals("You must specify either a nested [start] element or "
                + "the [starttarget] attribute", expected.getMessage());
        }
    }

    /**
     * Verifies that the <code>starttarget</code> is run before the timeout is 
     * exceeded. 
     * 
     * @throws Exception If an unexpected error occurs
     */
    public void testStartTimeout() throws Exception
    {
        try
        {
            executeTestTarget();
            fail("Expected BuildException");
        }
        catch (BuildException expected)
        {
            assertEquals("Failed to start the container after more than [0] "
                + "ms.", expected.getMessage());
        }
        assertTargetExecuted("startDummy");
    }

    /**
     * Verifies that a build exception is thrown if neither the
     * <code>stoptarget</code> nor a nested <code>stop</code> element has been
     * specified.
     * 
     * @throws Exception If an unexpected error occurs
     */
    public void testStopNotSet() throws Exception
    {
        try
        {
            executeTestTarget();
            fail("Expected BuildException");
        }
        catch (BuildException expected)
        {
            assertEquals("You must specify either a nested [stop] element or "
                + "the [stoptarget] attribute", expected.getMessage());
        }
    }

    /**
     * Verifies that a build exception is thrown if neither the
     * <code>testtarget</code> nor a nested <code>test</code> element has been
     * specified.
     * 
     * @throws Exception If an unexpected error occurs
     */
    public void testTestNotSet() throws Exception
    {
        try
        {
            executeTestTarget();
            fail("Expected BuildException");
        }
        catch (BuildException expected)
        {
            assertEquals("You must specify either a nested [test] element or "
                + "the [testtarget] attribute", expected.getMessage());
        }
    }

    /**
     * Verifies that a build exception is thrown if the <code>testurl</code>
     * attribute is not set.
     * 
     * @throws Exception If an unexpected error occurs
     */
    public void testTestUrlNotSet() throws Exception
    {
        try
        {
            executeTestTarget();
            fail("Expected BuildException");
        }
        catch (BuildException expected)
        {
            assertEquals("The [testurl] attribute must be specified",
                expected.getMessage());
        }
    }

    /**
     * Verifies that a build exception is thrown if the <code>testurl</code>
     * attribute is set to a non-HTTP URL.
     * 
     * @throws Exception If an unexpected error occurs
     */
    public void testNonHttpTestUrl() throws Exception
    {
        try
        {
            executeTestTarget();
            fail("Expected BuildException");
        }
        catch (IllegalArgumentException expected)
        {
            assertEquals("Not a HTTP URL", expected.getMessage());
        }
    }

}
